% script: runScenario
%
% Run a particular scenario and plot results (set up by "runSixScenarios.m")

% Copyright (c) 2016 by Gregory L. Plett of 
% University of Colorado Colorado Springs (UCCS). 
%
% This work is licensed under a Creative Commons 
% Attribution-NonCommercial-ShareAlike 4.0 Intl. License, v. 1.0
%
% It is provided "as is", without express or implied warranty, for 
% educational and informational purposes only.
%
% This file is provided as a supplement to: Plett, Gregory L., "Battery
% Management Systems, Volume II, Equivalent-Circuit Methods," Artech House, 
% 2015.

% Make up some data.
n = 1000; % number of data points to use in computations/plots
Q = (Q0+slope*(1:n))'; % true capacity as it (possibly) changes over time
x = ((xmax-xmin)*rand(n,1)+xmin);  % true change in SOC between measurements
y = Q.*x; % true accumulated ampere hours over that interval

% Add in some noise to both variables.
binsize = 2*maxI/precisionI; 
rn1 = ones(n,1);
if theCase == 1,
  rn2 = rn1;
  sy = binsize*sqrt(m/12)/3600*rn2;
else % requires "stats" toolbox, or else you will need to make your own
  mu = log(mode)+sigma^2;
  m = 3600*lognrnd(mu,sigma,n,1); 
% Uncomment to see pdf of lognormal pdf
%     figure; clf; hold on; 
%     plot(0:0.01:3,lognpdf(0:0.01:3,mu,sigma),'k');
%     xlabel('Time (hours)'); ylabel('PDF value');
%     title('PDF of drive cycle durations for EV scenarios'); box on;
  sy = binsize*sqrt(m/12)/3600;
end
sx = socnoise*rn1; 

x = x + sx.*randn(n,1); % add noise to true change in SOC 
y = y + sy.*randn(n,1); % add noise to accumulated ampere hours

%  - column 1 = WLS - weighted, recursive
%  - column 2 = WTLS - weighted, but not recursive
%  - column 3 = TLS - recursive weighted, but using SigmaX(1) and
%               SigmaY(1) only (i.e., simplified method using c0 and c1)
%  - column 4 = AWTLS - recursive and weighted
[Qhat,SigmaQ,Fit] = xLSalgos(x,y,sx.^2,sy.^2,gamma,Qnom);
Qrep = repmat(Q,1,size(Qhat,2));

figure; clf; hold on; plot(Qhat);
xlabel('Algorithm update index'); ylabel('Capacity estimate (Ah)');
title(sprintf('%s: Capacity Estimates with Error Bounds',plotParams.title));
legend('WLS','WTLS','TLS','AWTLS','location','northeast'); box on;
plot(Qhat+3*sqrt(SigmaQ),'linewidth',0.5);
plot(Qhat-3*sqrt(SigmaQ),'linewidth',0.5); plot(Qhat);
plot(1:n,Q,'k:','linewidth',1); 
ylim([min(plotParams.ytick) max(plotParams.ytick)]);
set(gca,'ytick',plotParams.ytick);

figure; clf; hold on; plot(Fit)
xlabel('Algorithm update index'); ylabel('Goodness of fit');
title(sprintf('%s: Goodness of fit for each method',plotParams.title))
legend('WLS','WTLS','TLS','AWTLS','location','east'); box on; 
ylim([-0.02 1.02]);

drawnow